/*
 * apGuard - Access Permissions Guard. 
 * Copyright (C) 2005-2006 Shobbo, urug
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include <unistd.h>	/* getopt */
#include <stdio.h>	/* puts   */
#include <string.h>	/* strcat */
#include "parser.h"
#include "perms.h"



/*
** Function prints help.
*/
static void
banner( void )
{
      puts("Usage: apguard <flags> configuration_file\n\n"
           "Available flags:\n"\
           "\t-l <file>\t: log file (default no logging)\n"\
           "\t-q\t\t: quiet.\n"\
	   "\t-d\t\t: debug.\n"\
	   "\t-h\t\t: this help.\n"\
	   "\t-p\t\t: parse config and exit.\n\n"\
	   "Please report bugs on https://sourceforge.net/projects/apguard/");

      _exit(1);
}



/*
** Clear command line options.
*/
static void
clear( COMMAND *cmd )
{
    cmd->log_file = NULL;
    cmd->only_parse = FALSE;
    cmd->debug = FALSE;
    cmd->quiet = FALSE;
}



/*
** Function runs parser and mod_perms.
*/
int
main( int argc, char **argv )
{
#   define	ABS_MAX 100
    INCLUDE	*inc = NULL;
    COMMAND	cmd;
    int		c;
    char	*cfg_file, absolute_pathname[ABS_MAX];
    boolean	status;


    /* Clear args. */
    clear(&cmd);

    /* Parse command line args. */
    while( (c = getopt( argc, argv, "qdphl:" ) ) > -1 )
        switch(c)
        {
            case 'l':	cmd.log_file = optarg; break;
            case 'p':	cmd.only_parse = TRUE; break;
            case 'd':	cmd.debug = TRUE; cmd.quiet = FALSE; break;
            case 'q':	cmd.quiet = TRUE; cmd.debug = FALSE; break;
            case 'h':	banner();
            case '?': 
            default:	return 1;
        }

    if( optind == argc )
        banner();

    /* Save config file path. */
    cfg_file = argv[optind];

    /* mod_perms() needs absolute log file path, because it sometimes uses chdir. */
    if( cmd.log_file != NULL && cmd.log_file[0] != '/' )
    {
          if( getcwd(absolute_pathname, sizeof(absolute_pathname)) == NULL )
          {
              puts("Sorry, but path to log file is too long. Specify absolute filename.");
              return 1;
          }
          else
          { 
              if( strlen(absolute_pathname) + strlen(cmd.log_file) < ABS_MAX )
              {
                  strcat(absolute_pathname, "/");
                  strcat(absolute_pathname, cmd.log_file);
              }
              else
              {
                  puts("Sorry, but path to log file is too long. Specify absolute filename.");
                  return 1;
              }
          }
    
          /* Point to new logfile directory. */
          cmd.log_file = absolute_pathname;
    }

    /* Parse config file and run mod_perms. */
    if( (status = p_init(&inc, cfg_file)) != FALSE && cmd.only_parse == FALSE )
        mod_perms(inc, &cmd);

    /* Free memory allocated by parser. */
    p_free(inc);

    /* Return 1 for error and 0 if all worked OK. */
    if( status == FALSE )
        return 1;
    else
        return 0;
}
