/*
 * parser.* - configuration file parser.
 * Copyright (C) 2005-2006 Shobbo, urug
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include <sys/types.h>  /* mode_t */
#include <sys/stat.h>   /* mode_t */
#include "misc.h"

#define NO_DEBUG		/* Debug parser ? */
#define STRING_MAX_SIZE	1024	/* Max number of characters in strings. */
#define OPTS_MAX_SIZE 	60	/* Max size of options attached to argument. */

/* Instruction types. */
#define i_VAR	5
#define i_INC	4


/* Struct stores all variables defined in config file. */
typedef struct var
{
    char	*name;          /* Variable name. */
    char	*contents;      /* Variable content. */
                        
    struct var  *next;
} VAR;


/* Structure describes string. */
typedef struct
{
    short   type;			/* Instruction type. */
    char    buffer[STRING_MAX_SIZE];	/* Bufer for rule. */
  
                                        /* FIRST/LAST SPACES ARE ALWAYS SKIPPED! */
    char    *list[2];			/* First/Last sign of filenames list. (If declared) */
    char    *name[2];			/* First/Last sign of filename/varname. */
    char    *args[2];			/* First/Last sign of args list/var content. */
} STRING;


/* Structure used in INCLUDE. */
typedef struct files
{
    char  *filename;
      
    struct files *next;
} FILES;


/* Files to exclude from include ;D */
typedef struct exclude
{
    char	*filename;
    
    struct exc_args
    {
        boolean		cur;
        boolean		sub;
        int		deep;
    } exc_args;

    struct exclude *next;
} EXCLUDE;


/* Structure contains informations about files to include. */
typedef struct include
{
    FILES	*include;
    EXCLUDE	*exclude;
    int	    	line_in_config;

    struct inc_args
    {
        boolean		check;
        boolean		change;
        boolean		cur;
        boolean		sub;
        int		deep;

        struct inc_perms
        {
            char	*user;
            char	*group;
            mode_t	mode;        
        } _cur, _sub;
    } inc_args;

    struct include *next;
} INCLUDE;


/* Functions created to manipulate config file parser. */
boolean	p_init( INCLUDE **inc_root, char *config );
void    p_free( INCLUDE *inc_root );
